#! /usr/bin/perl
# =======================================================================
#
#                   Jet Propulsion Laboratory
#
#                  Atmospheric InfraRed Sounder
#
#      Copyright (c) 2000 - California Institute of Technology
#              U.S. Government Sponsorship under NASA
#                Contract NAS7-1270 is acknowledged
# !PROLOG ==================================================================
#
#
#------------------------------------------------------------------------------
#
# 1. DISCLAIMER
#    THE SOFTWARE AND/OR RELATED MATERIALS ARE PROVIDED "AS-IS" WITHOUT
#    WARRANTY OF ANY KIND INCLUDING ANY WARRANTIES OF PERFORMANCE OR
#    MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE OR PURPOSE (AS SET FORTH
#    IN UCC 2312-2313) OR FOR ANY PURPOSE WHATSOEVER, FOR THE LICENSED
#    PRODUCT, HOWEVER USED.
#
#    IN NO EVENT SHALL CALTECH/JPL BE LIABLE FOR ANY DAMAGES AND/OR COSTS,
#    INCLUDING BUT NOT LIMITED TO INCIDENTAL OR CONSEQUENTIAL DAMAGES OF ANY
#    KIND, INCLUDING ECONOMIC DAMAGE OR INJURY TO PROPERTY AND LOST PROFITS,
#    REGARDLESS OF WHETHER CIT/JPL SHALL BE ADVISED, HAVE REASON TO KNOW,
#    OR IN FACT SHALL KNOW OF THE POSSIBILITY.
#
#    USER BEARS ALL RISK RELATING TO QUALITY AND PERFORMANCE OF THE SOFTWARE
#    AND/OR RELATED MATERIALS.
#
# 2. The user shall comply with all applicable U.S. export control laws and
#    regulations.  To the extent that the software is subject to U.S. export
#    control laws and regulations, the user has the responsibility to obtain
#    export licenses or other export authority as may be required before
#    exporting such information to foreign countries or providing access to
#    foreign persons.
#
#------------------------------------------------------------------------------
#
# !Routine/File Name:
#
#       airs_l2ir_hrepack.pl
#
#
# !Description:
#
#    This is a script to run within the Level 2 main PGE script (L2.ksh)
#    after the main executable has already created all product files.  This
#    script reads the file from LID 7301 (AIRS cloud cleared radiances) and
#    modifies it by chunking (one scan is one chunk) and compressing (GZIP 
#    at its fastest level) the "radiances" and "radiance_err" fields.  The
#    new file replaces the old one.
#
#    It returns 0 (success) unless L2 fails.
#
#
# !Input Parameters:
#
#    Number and names of granules to process are read in from the PCF file
#    (specified by $PGS_PC_INFO_FILE) from Logical ID (LID) 7301
#
# !Output Parameters:
#
#    None
#
# !Input/Output Parameters:
#
#    None
#
#
# !Global Variables:
#
#    None
#
#
# !Known Bugs and Limitations:
#
#    This is a wrapper for the HDF tool hrepack.  So there must be a
#    hrepack executable at $HDFBIN/hrepack.  This tool only became
#    part of HDF with release 4.2r0.
#
#
# !Revision History:  
#
#    First AIRS version made 2004-08-20 by Evan Manning as cut-down version
#    of GES DIS DAAC script s4pm_hrepack.pl.  AIRS version removes use
#    of an external script file, instead hardcoding LID/LUN number and
#    hrepack options.  This change relieves the scheduler from having to
#    supply the additional file, at cost of flexibility.
#
#
# !END ==================================================================

use strict ;
use Env ;
use Safe ;

my $status = 0 ;


#read in PCF
my %pcf = load_pcf($ENV{"PGS_PC_INFO_FILE"}) ;

# verify we have an hrepack to use
my $hrepack = $ENV{"HDFBIN"}."/hrepack" ;
die "hrepack executable ($hrepack) not found. Make sure HDFBIN ($HDFBIN) environment is set correctly." unless (-e $hrepack) ;

# dereference filenames from PCF, then call hrepack
foreach my $lun (7301) {
   foreach my $file (@{$pcf{$lun}}) {
      my $cmd = "$hrepack -i $file -o $file.$$.tmp "
      		."-t \'L2_Standard_cloud-cleared_radiance_product/Data Fields/radiances:GZIP 1\' "
      		."-t \'L2_Standard_cloud-cleared_radiance_product/Data Fields/radiance_err:GZIP 1\' "
		."-c \'L2_Standard_cloud-cleared_radiance_product/Data Fields/radiances:1x30x2378\' " 
		."-c \'L2_Standard_cloud-cleared_radiance_product/Data Fields/radiance_err:1x30x2378\'" ;
      # print "CMD: $cmd\n" ;
      my $ret = readpipe("$cmd") ;
      if ($ret ne "") {
         print "CMD: $cmd\nRETURN: $ret\n" ;
         unlink "$file.$$.tmp" ;
         $status = 1 ;
      } else {
         $status = rename "$file.$$.tmp",$file || $status ;
      }
   }
}

exit($status) ;

sub load_pcf {
  my $pgs_pc_info_file = shift ;
  my %pcf ;
  open (PCF,$pgs_pc_info_file) or die "Failure opening PGE_PC_INFO_FILE $pgs_pc_info_file\n" ;
  while (<PCF>) {
    next unless (/^\d+/) ;
    my @field = split /\|/ ;
    next unless ($#field > 2) ;
    $field[2] =~ s/\/+$// ;
    push @{$pcf{$field[0]}},"$field[2]/$field[1]" ;
  }
  close(PCF) ;
  return(%pcf) ;
}
